// This file is part of the PiDuce project (http://www.cs.unibo.it/Piduce/)
// developed at the Department of Computer Science of Bologna.
// Authors: Samuele Carpineti, Cosimo Laneve, Leonardo Mezzina, Luca Padovani
//
// See Copyright for the status of this software.

using System;
using System.Collections.Generic;
using System.Diagnostics;

namespace BoPi.Types
{
  public class EmptynessChecker
  {
    private IList<IType> eF = new List<IType>();
    private IList<IType> eT = new List<IType>();
    /// <summary>
    /// Checks if a given type is empty. (ErrorType is not empty by definition)
    /// </summary>
    /// <param name="s">the type to check for emptiness</param>
    /// <returns>true if <c>s</c> is empty, false otherwise</returns>
    public bool IsEmpty(IType s)
    { 
      bool res = IsEmpty(s, new List<IType>());
      if (res) eT.Add(s);
      else eF.Add(s);
      return res;
    }
    
    private bool IsEmpty(IType s, IList<IType> visited)
    {
      if (s.IsBaseType()) return false;
      else if (s.IsVoid()) return false;
      else if (s.IsChan()) return false;
      else if (s.IsBottom()) return true;
      else if (s.IsErrorType()) return false;
      else if (eT.Contains(s)) return true;
      else if (eF.Contains(s)) return false;
      else
      {
        bool res = false;  
        if (s.IsLabelled()) 
          res = s.AsLabelled().Labels.IsEmpty() || 
            IsEmpty(s.AsLabelled().Content, visited);
        else if (s.IsSequence()) 
          res = IsEmpty(s.AsSequence().Top, visited) || 
            IsEmpty(s.AsSequence().Tail, visited);
        else if (s.IsUnion()) 
          res = IsEmpty(s.AsUnion().Fst, visited) && 
            IsEmpty(s.AsUnion().Snd, visited);
        else if (s.IsConstantTypeName())
        {
          if (visited.Contains(s)) return true;
          else
          {
            visited.Add(s);
            res = IsEmpty(s.AsConstantTypeName().Entry.Type, visited);
          }
        }
        else throw new ApplicationException("public static bool IsEmpty: This code should be unreacheable");
        return res;
      }
    }
  }
}
