// This file is part of the PiDuce project (http://www.cs.unibo.it/Piduce/)
// developed at the Department of Computer Science of Bologna.
// Authors: Samuele Carpineti, Cosimo Laneve, Leonardo Mezzina, Luca Padovani
//
// See Copyright for the status of this software.

using System;
using BoPi.Common;
using System.Xml;

namespace BoPi.Types
{
	public interface ITypeVisitor
	{
		void VisitBottom(Bottom t);
		void VisitVoid(Void t);
		void VisitUnion(Union t);
		void VisitConstantTypeName(ConstantTypeName t);
		void VisitSequence(Sequence t);
		void VisitLabelled(Labelled t);
		void VisitIntLiteral(IntLiteral t);
		void VisitIntType(IntType t);
		void VisitStringLiteral(StringLiteral t);
		void VisitStringType(StringType t);
		void VisitChan(Chan t);
		void VisitErrorType(ErrorType t);
		void VisitBind(Bind t);
	}

  ///<summary>Interface of BoPi types</summary>
  public interface IType
  {	
    bool IsInt();
    bool IsString();
    bool IsIntLiteral();
    bool IsStringLiteral();
    bool IsLabelled();
    bool IsSequence();
    bool IsUnion();
    bool IsConstantTypeName();
    bool IsBaseType();
    bool IsChan();
    bool IsVoid();
    bool IsBottom();
    bool IsErrorType();
    IntType AsInt();
    StringType AsString();
    IntLiteral AsIntLiteral();
    StringLiteral AsStringLiteral();
    Labelled AsLabelled();
    Sequence AsSequence();
    Union AsUnion();
    ConstantTypeName AsConstantTypeName();
    Chan AsChan();
    Void AsVoid();
    Bottom AsBottom();
    /// <summary>
    /// A type is equal to an object only if the object is a type having the same structure of this type 
    /// </summary>
    /// <param name="obj">the object to compare with</param>
    /// <returns></returns>
    String ToString();
		IType Strip();
		void ResolveNames(TypeSymbolTable st);
    void Compile(XmlWriter xml);
		void Accept(ITypeVisitor visitor);
  }
}
